<?php
 /**************************************************************************\
 * eGroupWare - correspondence                                              *
 * http://www.egroupware.org                                                *
 * -----------------------------------------------                          *
 *  This program is free software; you can redistribute it and/or modify it *
 *  under the terms of the GNU General Public License as published by the   *
 *  Free Software Foundation; either version 2 of the License, or (at your  *
 *  option) any later version.                                              *
 \**************************************************************************/

class bocorrespondence
{
  var $return_to;
  
  var $owner;
  var $prefs;

  var $filter;
  var $filter_dept;
  var $cat_id;

  var $use_session = False;
  var $db;

  var $public_functions = array(
				'delete' => True,
				'update' => True,
				'update_attachments' => True,
				'get_list' => True,
				'read_repository' => True,
				'open_attachment' => True,
				'get_attachments_list' => True,
				'add_history' => True,
				'get_adv_search' => True,
				'async_notification' => True
				);

  function bocorrespondence($session=0)
  {
    $this->cat = CreateObject('phpgwapi.categories');
    $this->status = CreateObject('correspondence.status');

    if($session) {
      $this->read_sessiondata();
      $this->use_session = True;
    }

    $this->prefs['correspondence']= $GLOBALS['egw_info']['user']['preferences']['correspondence'];
    $this->check_set_default_prefs();

    $owner = get_var('owner',array('GET','POST'),$GLOBALS['owner']);
    if(isset($owner) && $owner!='') {
      $this->owner = intval($owner);
    }
    elseif(!@isset($this->owner) || !@$this->owner) {
      $this->owner = intval($GLOBALS['egw_info']['user']['account_id']);
    }

    if(isset($_POST['filter'])) { $this->filter = $_POST['filter']; }
    if(isset($_POST['cat_id'])) { $this->cat_id = $_POST['cat_id']; }
    if(isset($_POST['dir_id'])) { $this->direction = $_POST['dir_id']; }
    if(isset($_POST['filter_dept'])) { $this->filter_dept = $_POST['filter_dept']; }
    
    if(!isset($this->filter)) {
      $this->filter = $this->prefs['correspondence']['defaultfilter'];
    }

    if(!isset($this->direction)) {
      $this->direction = $this->prefs['correspondence']['defaultbox'];
    }
    
    $this->db = $GLOBALS['egw']->db;
  }

  /**
  * Send all async correspondence notifications
  * Called via the async service job 'correspondence-async-notification'
  */
  function async_notification()
  {
    $today = time();
    $first_notify_days = 5;
    $second_notify_days = 1;

    // early notification
    $whereclause = "WHERE deadline>".intval($today+($first_notify_days-1)*24*3600)." AND deadline<".intval($today+$first_notify_days*24*3600);
    $sql = "SELECT * FROM phpgw_correspondence $whereclause";
    $this->db->query($sql,__LINE__,__FILE__);

    while ($this->db->next_record()) {
      $doc_protocol = $this->db->f('id_protocol');
      $doc_deadline = $this->db->f('deadline');
      $doc_subject = $this->db->f('subject');
      $doc_watchers = $this->db->f('watchers');
      $doc_to = $this->db->f('doc_to');

      // send email for members of To field
      $l_to_members = $GLOBALS['egw']->accounts->members($doc_to, true);
      if ($l_to_members != '') {
        $return = $this->send_notification_deadline($l_to_members,$first_notify_days,$doc_protocol,$doc_deadline,$doc_subject);
      }

      // send email for extra participants
      if ($doc_watchers != '') {
        $return = $this->send_notification_deadline(explode(',',$doc_watchers),$first_notify_days,$doc_protocol,$doc_deadline,$doc_subject);
      }
    }

    // late notification
    $whereclause = "WHERE deadline>".intval($today+($second_notify_days-1)*24*3600)." AND deadline<".intval($today+$second_notify_days*24*3600);
    $sql = "SELECT * FROM phpgw_correspondence $whereclause";
    $this->db->query($sql,__LINE__,__FILE__);

    while ($this->db->next_record()) {
      $doc_protocol = $this->db->f('id_protocol');
      $doc_deadline = $this->db->f('deadline');
      $doc_subject = $this->db->f('subject');
      $doc_watchers = $this->db->f('watchers');
      $doc_to = $this->db->f('doc_to');

      // send email for members of To field
      $l_to_members = $GLOBALS['egw']->accounts->members($doc_to, true);
      if ($l_to_members != '') {
        $return = $this->send_notification_deadline($l_to_members,$second_notify_days,$doc_protocol,$doc_deadline,$doc_subject);
      }

      // send email for extra participants
      if ($doc_watchers != '') {
        $return = $this->send_notification_deadline(explode(',',$doc_watchers),$second_notify_days,$doc_protocol,$doc_deadline,$doc_subject);
      }
    }
 
    return;
  }

  function check_perms($has, $needed)
  {
    return (!!($has & $needed) == True);
  }
  
  function check_set_default_prefs()
  {
    if (($set = $GLOBALS['egw']->session->appsession('default_prefs_set','correspondence'))) {
      return;
    }
    $GLOBALS['egw']->session->appsession('default_prefs_set','correspondence','set');
    
    $default_prefs = $GLOBALS['egw']->preferences->default['correspondence'];
    
    $defaults = array(
      'defaultcategory'  => 'all',
      'defaultfilter'    => 'all',
      'defaultbox'       => 'R',
      'near_days'        => '5',
      'default_deadline' => '10',
      'receive_updates'  => 'no',
      'defaultsort'      => 'deadline',
      'defaultsortdir'   => 'ASC'
    );

    foreach($defaults as $var => $default) {
      if (!isset($default_prefs[$var]) || $default_prefs[$var] == '') {
	$GLOBALS['egw']->preferences->add('correspondence',$var,$default,'default');
	$need_save = True;
      }
    }

    if ($need_save) {
      $prefs = $GLOBALS['egw']->preferences->save_repository(False,'default');
      $this->prefs['correspondence'] = $prefs['correspondence'];
    }
  }

  function save_sessiondata($data='')
  {
    if ($this->use_session) {
      if (!is_array($data)) {
	$data = array(
		      'filter'     	=> $this->filter,
		      'filter_dept'     => $this->filter_dept,
		      'cat_id'     	=> $this->cat_id,
		      'direction'  	=> $this->direction,
		      'owner'      	=> $this->owner,
		      'return_to'  	=> $this->return_to
		      );
      }
      $GLOBALS['egw']->session->appsession('session_data','correspondence',$data);
    }
  }

  function read_sessiondata()
  {
    $data = $GLOBALS['egw']->session->appsession('session_data','correspondence');
    
    $this->filter = $data['filter'];
    $this->filter_dept = $data['filter_dept'];
    $this->direction = $data['direction'];
    $this->cat_id = $data['cat_id'];
    $this->owner  = intval($data['owner']);
    $this->return_to = $data['return_to'];
  }

  function read_repository($doc_id)
  {
    $this->db->query('SELECT * FROM phpgw_correspondence WHERE doc_id='
		     . intval($doc_id),__LINE__,__FILE__);
    $this->db->next_record();

    $data['subject']     = $this->db->f('subject');
    $data['owner']       = $this->db->f('owner');
    $data['status']      = $this->db->f('status');
    $data['protocol']    = $this->db->f('id_protocol');
    $data['id_archive']  = $this->db->f('id_archive');
    $data['priority']    = $this->db->f('priority');
    $data['description'] = $this->db->f('description');
    $data['category']    = $this->db->f('category');
    $data['watchers']    = $this->db->f('watchers');
    $data['doc_from']    = $this->db->f('doc_from');
    $data['doc_to']      = $this->db->f('doc_to');
    $data['doc_type']    = $this->db->f('doc_type');
    $data['startdate']   = $this->db->f('startdate');
    $data['deadline' ]   = $this->db->f('deadline');
    $data['direction']   = $this->db->f('direction');

    return $data;
  }

  function get_list($start = -1, $sort = '', $order = '', $query = '', $offset = -1)
  {
    if (! $sort) {
      $sort = "DESC";
    }
    
    if ($order) {
      $orderclause = "ORDER BY $order $sort";
    }
    else {
      $orderclause = "ORDER BY doc_id ASC";
    }

    if ($this->filter == 'concluded') {
      $whereclause = "WHERE status='".$this->filter."' ";
    }
    elseif ($this->filter == 'pending') {
      $whereclause = "WHERE status='pending' AND " . '(deadline>'.time().' OR deadline=0) ';
    }
    elseif ($this->filter == 'expired') {
      $whereclause = "WHERE status='pending' AND deadline<".time()." ";
    }

    if ($this->direction) {
      if ($whereclause) {
	$whereclause .= 'AND ';
      }
      else {
	$whereclause .= 'WHERE ';
      }

      $whereclause .= "direction='".$this->direction."' ";
    }

    if ($this->cat_id) {
      if ($whereclause) {
	$whereclause .= 'AND ';
      }
      else {
	$whereclause .= 'WHERE ';
      }

      $whereclause .= "category LIKE '%".$this->cat_id."%' ";
    }

    if ($this->filter_dept) {
      if ($whereclause) {
	$whereclause .= 'AND ';
      }
      else {
	$whereclause .= 'WHERE ';
      }

      $GLOBALS['egw']->accounts->get_account_name($this->filter_dept,$dept_name,$tmp1,$tmp2);
      $whereclause .= "doc_to LIKE '%".$dept_name."%' ";
    }

    if ($query) {
      $query = $this->db->db_addslashes($query);

      if ($whereclause) {
	$whereclause .= 'AND ';
      }
      else {
	$whereclause .= 'WHERE ';
      }

      $whereclause .= "(id_protocol LIKE '%$query%' OR doc_from LIKE '%$query%' OR doc_to LIKE " . "'%$query%' OR subject LIKE '%$query%' OR description LIKE '%$query%')";
    }

    $sql = "SELECT * FROM phpgw_correspondence $whereclause $orderclause";
    if ($offset == -1 && $start == -1) {
      $this->db->query($sql,__LINE__,__FILE__);
    } 
    elseif ($start != -1) {
      $this->db->limit_query($sql,$start,__LINE__,__FILE__);
    }
    else {
      $this->db->limit_query($sql,$start,__LINE__,__FILE__,$offset);
    }
    
    while ($this->db->next_record()) {
      $data[] = Array(
			  'doc_id'    => $this->db->f('doc_id'),
			  'protocol'  => $this->db->f('id_protocol'),
			  'doc_from'  => $this->db->f('doc_from'),
			  'doc_to'    => $this->db->f('doc_to'),
			  'startdate' => $this->db->f('startdate'),
			  'deadline'  => $this->db->f('deadline'),
			  'owner'     => $this->db->f('owner'),
			  'doc_type'  => $this->db->f('doc_type'),
			  'subject'   => $this->db->f('subject'),
			  'status'    => $this->db->f('status'),
			  'priority'  => $this->db->f('priority')
			  );
    }
    $this->db->query("SELECT count(*) FROM phpgw_correspondence $whereclause");
    $this->db->next_record();
    $this->total = $this->db->f(0);
    
    return $data;
  }

  function delete($doc_id)
  {
    if ($doc_id) {
      $this->db->query('DELETE FROM phpgw_correspondence WHERE doc_id='.$doc_id,
		       __LINE__,__FILE__);      
    }

    // remove attachments
    $attach_dir = $GLOBALS['egw_info']['server']['files_dir'].SEP.'correspondence'.SEP.$doc_id;
    if (file_exists($attach_dir)) {
      $dh = opendir($attach_dir);
      while ($file = readdir($dh)) {
	if (($file != '.') && ($file != '..')) {
	  unlink($attach_dir.SEP.$file);
	}
      }
      closedir($dh);
      rmdir($attach_dir);
    }
    
    $this->db->query('DELETE FROM phpgw_correspondence_attachs WHERE doc_id='.$doc_id,
		     __LINE__,__FILE__);      

    // remove history
    $this->db->query('DELETE FROM phpgw_correspondence_hist WHERE doc_id='.$doc_id,
		     __LINE__,__FILE__);      


    Header('Location: '.$GLOBALS['egw']->link('/index.php','menuaction=correspondence.uicorrespondence.listview'));
    $GLOBALS['egw']->common->phpgw_exit();
  }

  function fix_update_time(&$time_param)
  {
    if (isset($time_param['str'])) {
      if (!is_object($this->jscal)) {
	$this->jscal = CreateObject('phpgwapi.jscalendar');
      }
      $time_param += $this->jscal->input2date($time_param['str'],False,'mday');
      unset($time_param['str']);
    }
  }
 
  function update()
  {
    if ($_POST['submit']) {
      $l_subject = $_POST['subject'];
      $l_proto = $_POST['protocol'];
      $l_description = $_POST['description'];
      $l_categories = $_POST['categories'];
      $l_types = $_POST['types'];
      $l_direction = $_POST['inout'];
      $l_from = $_POST['from'];
      $l_date = $_POST['datestart'];

      if (isset($_POST['watchers'])) {
        $l_watchers = $_POST['watchers'];
      }

      unset($l_to_members);
      if (isset($_POST['doc_to']) && $_POST['doc_to'] != '0' && $_POST['doc_to'] != '') {
        $l_to = $_POST['doc_to'];
	$l_to_members = $GLOBALS['egw']->accounts->members($l_to, true);
      } else {
        $l_to = isset($_POST['doc_to_other'])?$_POST['doc_to_other']:'';
      }

      if (isset($_POST['deadline'])) {
	$deadline = $_POST['deadline'];
	if ($deadline['str'] != '') {
	  $l_deadline = $deadline;
	}
      }

      $l_priority = isset($_POST['priority'])?$_POST['priority']:'0';
      $l_status = isset($_POST['status'])?$_POST['status']:'pending';
    }

    if(isset($_POST['doc_id'])) {
      $doc_id = $_POST['doc_id'];
    } else {
      $doc_id = 0;
    }

    $tmp_doc_id = $_POST['tmp_doc_id']?intval($_POST['tmp_doc_id']):0;

    $this->fix_update_time($l_date);
    $start = mktime(0,0,0,$l_date['month'],$l_date['mday'],$l_date['year']) - $GLOBALS['egw']->datetime->tz_offset;

    if (isset($l_deadline)) {
      $this->fix_update_time($l_deadline);
      $deadline = mktime(0,0,0,$l_deadline['month'],$l_deadline['mday'],$l_deadline['year']) - $GLOBALS['egw']->datetime->tz_offset;
    } else {
      $deadline = 0;
    }
 
    if(count($l_categories) >= 2) {
      $category = implode(',',$l_categories);
    }
    else {
      $category = strval($l_categories[0]);
    }

    if(count($l_watchers) >= 2) {
      $watchers = implode(',',$l_watchers);
    }
    elseif(count($l_watchers) == 1) {
      $watchers = strval($l_watchers[0]);
    }
    else {
      $watchers = '';
    }

    $doc_type = strval($l_types[0]);

    // prepare attachments
    if ($doc_id > 0) {
      $upload_dir = $GLOBALS['egw_info']['server']['files_dir'].SEP.'correspondence'.SEP.$doc_id;
    } elseif ($tmp_doc_id > 0) {
      $upload_dir = $GLOBALS['egw_info']['server']['files_dir'].SEP.'correspondence'.SEP.'T'.$tmp_doc_id;
    }
    else {
      return False;
    }

    if (file_exists($upload_dir)) {
      $dh = opendir($upload_dir);
      while ($file = readdir($dh)) {
	if (($file != '.') && ($file != '..') && (! ereg("\.info",$file))) {
	  $file_size = filesize($upload_dir.SEP.$file);
	  
	  $info_file = $upload_dir.SEP.$file.'.info';
	  $file_info = file($info_file);
	  $file_name = trim($file_info[1]);
	  
	  // insert into array
	  $a_files[] = array(
			     'name'     => $file_name,
			     'int_name' => $file,
			     'size'     => $file_size
			     );
	}
      }
      closedir($dh);
    }
    

    // new record
    if ($doc_id == 0) {

      // if protocol number is empty or zero, simply autoincrement
      if ( ($l_proto == '') || ($l_proto == 0) ) {
        $this->db->query("SELECT max(id_protocol) FROM phpgw_correspondence");
        $this->db->next_record();
        $l_proto = $this->db->f(0) + 1;
      }

      $this->db->query('INSERT INTO phpgw_correspondence (subject, '
		       . ' description, category, direction, doc_type, '
		       . ' doc_from, doc_to, id_protocol, deadline, '
		       . ' priority, status, startdate, watchers, '
		       . " owner) VALUES ("
		       . "'" . $this->db->db_addslashes($l_subject) . "',"
		       . "'" . $this->db->db_addslashes($l_description) . "',"
		       . "'" . $this->db->db_addslashes($category) . "',"
		       . "'" . $this->db->db_addslashes($l_direction) . "',"
		       . "'" . $this->db->db_addslashes($doc_type) . "',"
		       . "'" . $this->db->db_addslashes($l_from) . "',"
		       . "'" . $this->db->db_addslashes($l_to) . "',"
		       . intval($l_proto) . ","
		       . intval($deadline) . ","
		       . "'" . $this->db->db_addslashes($l_priority) . "',"
		       . "'" . $this->db->db_addslashes($l_status) . "',"
		       . intval($start) . ","
		       . "'" . $this->db->db_addslashes($watchers) . "',"
		       . intval($this->owner)
		       . ')',__LINE__,__FILE__);

      $doc_id = $this->db->get_last_insert_id('phpgw_correspondence','doc_id');

      // send email for members of To field
      if ($l_to_members != '') {
	$return = $this->send_notification($l_to_members,$doc_id);
      }

      // send email for all participants
      if ($watchers != '') {
	$return = $this->send_notification($l_watchers,$doc_id);
      }

      // rename attachments directory to definitive name add
      // references to database
      if (count($a_files) > 0) {
	$final_upload_dir = $GLOBALS['egw_info']['server']['files_dir'].SEP.'correspondence'.SEP.$doc_id;
	$result = rename($upload_dir,$final_upload_dir);
	if (! $result) {
	  return False;
	}

	while (list(,$file) = each($a_files)) {
	  $this->db->query('INSERT INTO phpgw_correspondence_attachs (doc_id, '
			   . "name, int_name, size) VALUES ("
			   . intval($doc_id) . ','
			   . "'" . $this->db->db_addslashes($file['name']) . "',"
			   . "'" . $this->db->db_addslashes($file['int_name']) . "',"
			   . intval($file['size'])
			   . ')',__LINE__,__FILE__);
	}
      }
    }
    else {
      $this->db->query('UPDATE phpgw_correspondence SET'
       . " subject='" . $this->db->db_addslashes($l_subject) . "',"
       . " description='" . $this->db->db_addslashes($l_description) . "',"
       . " category='" . $this->db->db_addslashes($category) . "',"
       . " watchers='" . $this->db->db_addslashes($watchers) . "',"
       . " direction='" . $this->db->db_addslashes($l_direction) . "',"
       . " doc_type='" . $this->db->db_addslashes($doc_type) . "',"
       . " doc_from='" . $this->db->db_addslashes($l_from) . "',"
       . " doc_to='" . $this->db->db_addslashes($l_to) . "',"
       . " id_protocol='" . intval($l_proto) . "',"
       . ' priority=' . intval($l_priority) . ','
       . " status='" . $this->db->db_addslashes($l_status) . "',"
       . ' startdate=' . intval($start) . ','
       . ' deadline=' . intval($deadline)
       . ' WHERE doc_id='.$doc_id,__LINE__,__FILE__);

      // remove old attachments
      // FIX: needs more efficient way to update attachments
      $this->db->query('DELETE FROM phpgw_correspondence_attachs WHERE doc_id='.$doc_id,
		       __LINE__,__FILE__);
    }

    // update attachments to another table
    if (count($a_files) > 0) {
      while (list(,$file) = each($a_files)) {
	$this->db->query('INSERT INTO phpgw_correspondence_attachs (doc_id, '
		 . "name, int_name, size) VALUES ("
		 . intval($doc_id) . ','
		 . "'" . $this->db->db_addslashes($file['name']) . "',"
		 . "'" . $this->db->db_addslashes($file['int_name']) . "',"
		 . intval($file['size'])
		 . ')',__LINE__,__FILE__);
      }
    }

    $this->read_sessiondata();


    if ($this->return_to) {
     $GLOBALS['egw']->redirect_link('/index.php','menuaction='.$this->return_to);
     $GLOBALS['egw']->common->phpgw_exit();
    }
    else {
     Header('Location: '.$GLOBALS['egw']->link('/index.php','menuaction=correspondence.uicorrespondence.listview'));
     $GLOBALS['egw']->common->phpgw_exit();
    }
  }

  function update_attachments()
  {
    if(isset($_GET['doc_id'])) {
      $doc_id = $_GET['doc_id'];
    } else {
      $doc_id = 0;
    }

    // prepare attachments
    if ($doc_id > 0) {
      $upload_dir = $GLOBALS['egw_info']['server']['files_dir'].SEP.'correspondence'.SEP.$doc_id;
    } else {
      return False;
    }

    if (file_exists($upload_dir)) {
      $dh = opendir($upload_dir);
      while ($file = readdir($dh)) {
	if (($file != '.') && ($file != '..') && (! ereg("\.info",$file))) {
	  $file_size = filesize($upload_dir.SEP.$file);
	  
	  $info_file = $upload_dir.SEP.$file.'.info';
	  $file_info = file($info_file);
	  $file_name = trim($file_info[1]);
	  
	  // insert into array
	  $a_files[] = array(
			     'name'     => $file_name,
			     'int_name' => $file,
			     'size'     => $file_size
			     );
	}
      }
      closedir($dh);
    }
    
    // remove old attachments
    // FIX: needs more efficient way to update attachments
    $this->db->query('DELETE FROM phpgw_correspondence_attachs WHERE doc_id='.$doc_id,__LINE__,__FILE__);

    // update attachments to another table
    if (count($a_files) > 0) {
      while (list(,$file) = each($a_files)) {
	$this->db->query('INSERT INTO phpgw_correspondence_attachs (doc_id, '
		 . "name, int_name, size) VALUES ("
		 . intval($doc_id) . ','
		 . "'" . $this->db->db_addslashes($file['name']) . "',"
		 . "'" . $this->db->db_addslashes($file['int_name']) . "',"
		 . intval($file['size'])
		 . ')',__LINE__,__FILE__);
      }
    }

    $this->read_sessiondata();
    if ($this->return_to) {
      $GLOBALS['egw']->redirect_link('/index.php',
	       array(
		     'menuaction' => $this->return_to,
		     'doc_id' => $doc_id
		     ));
      $GLOBALS['egw']->common->phpgw_exit();
    }
    Execmethod('correspondence.uicorrespondence.listview');
  }

  function get_attachments_names($doc_id)
  {
    $this->db->query('SELECT * FROM phpgw_correspondence_attachs WHERE doc_id='
		     . intval($doc_id),__LINE__,__FILE__);
    
    while ($this->db->next_record()) {
      $data[] = $this->db->f('name');
    }
    
    return $data;
  }

  function get_attachments_list($doc_id)
  {
    $this->db->query('SELECT * FROM phpgw_correspondence_attachs WHERE doc_id='
		     . intval($doc_id),__LINE__,__FILE__);
    while ($this->db->next_record()) {
      $data[] = array(
		      'attach_id' => $this->db->f('attach_id'),
		      'name'      => $this->db->f('name'),
		      'int_name'  => $this->db->f('int_name'),
		      'size'      => $this->db->f('size')
		      );
    }
    
    return $data;
  }

  function open_attachment()
  {
    $egw_flags = Array(
			 'noheader'      =>      True,
			 'nofooter'      =>      True,
			 'nonavbar'      =>      True
			 );
    
    $GLOBALS['egw_info']['flags'] = $egw_flags;
    
    if(isset($_GET['attach_id'])) {
      $attach_id = $_GET['attach_id'];
    } else {
      // FIX: need better error treatment and response
      return False;
    }

    $this->db->query('SELECT * FROM phpgw_correspondence_attachs WHERE attach_id='
		     . intval($attach_id),__LINE__,__FILE__);
    $this->db->next_record();

    $doc_id = $this->db->f('doc_id');
    $int_name = $this->db->f('int_name');
    
    $fullpath = $GLOBALS['egw_info']['server']['files_dir'].SEP.'correspondence'.SEP.$doc_id.SEP.$int_name;

    $info_file = $fullpath.'.info';
    $file_info = file($info_file);
    $file_type = trim($file_info[0]);
    $file_name = trim($file_info[1]);

    Header("Cache-Control: private");
    Header("Content-type: ".$file_type);
    Header('Content-Disposition: attachment; filename="'.$file_name.'"');

    $fh = fopen($fullpath,'rb');
    while ($datachunk = fread($fh, 256)) {
      echo $datachunk;
    }
    fclose($fh);
  }

  function get_history_list($doc_id)
  {
    $orderclause = " ORDER BY date ASC";
    
    $this->db->query('SELECT * FROM phpgw_correspondence_hist WHERE doc_id='
		     . intval($doc_id) . $orderclause,__LINE__,__FILE__);
    while ($this->db->next_record()) {
      $data[] = array(
		      'date'     => $this->db->f('date'),
		      'user'     => $this->db->f('c_user'),
		      'fw_user'  => $this->db->f('fw_user'),
		      'notes'    => $this->db->f('notes')
		      );
    }
    
    return $data;
  }

  function get_last_history($doc_id)
  {
    $orderclause = " ORDER BY date DESC LIMIT 1";
    
    $this->db->query('SELECT * FROM phpgw_correspondence_hist WHERE doc_id='
		     . intval($doc_id) . $orderclause,__LINE__,__FILE__);
    while ($this->db->next_record()) {
      $data = array(
		    'date'     => $this->db->f('date'),
		    'user'     => $this->db->f('c_user'),
		    'fw_user'  => $this->db->f('fw_user'),
		    'notes'    => $this->db->f('notes')
		    );
    }
    
    return $data;
  }

  function add_history()
  {
    if ($_POST['submit']) {
      if ($_POST['watchers']) {
	$l_forward = $_POST['watchers'];
      }
      if ($_POST['archiveid']) {
	$l_archiveid = $_POST['archiveid'];
      }
      $l_notes = $_POST['notes'];
      $l_user = $GLOBALS['egw_info']['user']['account_id'];
      $doc_id  = $_POST['doc_id'];
    }
    
    $today = time();
    
    if ($l_archiveid) {
      $this->db->query('UPDATE phpgw_correspondence SET'
		       . " status='concluded',"
		       . ' id_archive=' . intval($l_archiveid)
		       . ' WHERE doc_id='.$doc_id,__LINE__,__FILE__);
    }
    
    if ($l_forward) {
      // insert into database
      if(count($l_forward) >= 2) {
	$fw_user = implode(',',$l_forward);
      }
      else {
	$fw_user = strval($l_forward[0]);
      }
 
      // send e-mail
      $return = $this->send_history_update($l_forward,$doc_id,$l_notes);
      if ($return) {
	$this->db->query('INSERT INTO phpgw_correspondence_hist (doc_id, '
			 . 'date, c_user, fw_user, notes) '
			 . "VALUES ("
			 . intval($doc_id) . ","
			 . intval($today) . ","
			 . intval($l_user) . ","
			 . "'" . $fw_user . "',"
			 . "'" . $this->db->db_addslashes($l_notes)
			 . "')",__LINE__,__FILE__);
      }
    }	  
    else {
      $this->db->query('INSERT INTO phpgw_correspondence_hist (doc_id, date, '
		       . 'c_user, notes) '
		       . "VALUES ("
		       . intval($doc_id) . ","
		       . intval($today) . ","
		       . intval($l_user) . ","
		       . "'" . $this->db->db_addslashes($l_notes)
		       . "')",__LINE__,__FILE__);
    }

    $this->read_sessiondata();
    if ($this->return_to) {
      $GLOBALS['egw']->redirect_link('/index.php',
	       array(
		     'menuaction' => $this->return_to,
		     'doc_id' => $doc_id
		     ));
      $GLOBALS['egw']->common->phpgw_exit();
    }
    Execmethod('correspondence.uicorrespondence.listview');
  }

  function date2string($_date)
  {
    $dateformat = $GLOBALS['egw_info']['user']['preferences']['common']['dateformat'];
    $d = getdate($_date);
    $datestring = date($dateformat,
		       mktime(0,0,0,$d['mon'],$d['mday'],$d['year']));

    return $datestring;
  }

  function send_history_update($to_notify,$doc_id,$notes)
  {
    // provide information to recipient
    $doc_info = $this->read_repository($doc_id);
    
    $user = $GLOBALS['egw_info']['user']['account_id'];
    $sender = $GLOBALS['egw_info']['user']['preferences']['email']['address'];
    if(!is_object($GLOBALS['egw']->send)) {
      $GLOBALS['egw']->send = CreateObject('phpgwapi.send');
    }
    $send = &$GLOBALS['egw']->send;
    
    foreach($to_notify as $userid) {
      if($GLOBALS['egw']->accounts->get_type($userid) == 'g') {
        continue;
      }
      $userid = intval($userid);
      
      $preferences = CreateObject('phpgwapi.preferences',$userid);
      $GLOBALS['egw']->accounts->get_account_name($userid,$lid,$to_firstname,$to_lastname);
      $to_fullname = $GLOBALS['egw']->common->display_fullname('',$to_firstname,$to_lastname);
      
      $to = $preferences->email_address($userid);
      if (empty($to) || $to[0] == '@' || $to[0] == '$') {
	// we have no valid email-address
	continue;
      }
      
      $subject = lang('notification message for your correspondence')." - ".$doc_info['subject'];
      $subject = trim($send->encode_subject($subject));
      
      $deadline = $this->date2string($doc_info['deadline']);

      $body = lang('a correspondence requires your attention.');
      $body .= ' '.lang('details follow').":\n";
      $body .= "\n".lang('protocol number').': '.$doc_info['protocol']."\n";
      $body .= lang('subject').': '.$doc_info['subject']."\n";
      $body .= lang('deadline').': '.$deadline."\n\n";
      $body .= lang('the following note was included by the sender').":\n";
      $body .= "\n".$notes;
      
      $returncode = $send->msg('email',$to,$subject,$body,'','','',$sender);
      
      if (!$returncode) {
	echo '<p><b>send_history_update</b>: '.lang("Failed sending message to '%1' #%2 subject='%3', sender='%4' !!!",$to,$userid,htmlspecialchars($subject), $sender)."<br />\n";
	echo '<i>'.$send->err['desc']."</i><br />\n";
	echo lang('This is mostly caused by a not or wrongly configured SMTP server. Notify your administrator.')."</p>\n";
	echo '<p>'.lang('Click %1here%2 to return to Correspondence.','<a href="'.$GLOBALS['egw']->link('/correspondence/').'">','</a>')."</p>\n";
      }
    }
    unset($send);
    
    return $returncode;
  }

  function send_notification($to_notify,$doc_id)
  {
    // provide information to recipient
    $doc_info = $this->read_repository($doc_id);
    
    $user = $GLOBALS['egw_info']['user']['account_id'];
    $sender = $GLOBALS['egw_info']['user']['preferences']['email']['address'];
    if(!is_object($GLOBALS['egw']->send)) {
      $GLOBALS['egw']->send = CreateObject('phpgwapi.send');
    }
    $send = &$GLOBALS['egw']->send;
    
    foreach($to_notify as $userid) {
      if($GLOBALS['egw']->accounts->get_type($userid) == 'g') {
	continue;
      }
      $userid = intval($userid);
      
      $preferences = CreateObject('phpgwapi.preferences',$userid);
      $GLOBALS['egw']->accounts->get_account_name($userid,$lid,$to_firstname,$to_lastname);
      $to_fullname = $GLOBALS['egw']->common->display_fullname('',$to_firstname,$to_lastname);
      
      $to = $preferences->email_address($userid);
      if (empty($to) || $to[0] == '@' || $to[0] == '$') {
	// we have no valid email-address
	continue;
      }
      
      $subject = lang('notification message for a new correspondence')." - ".$doc_info['subject'];
      $subject = trim($send->encode_subject($subject));
      
      $deadline = $this->date2string($doc_info['deadline']);

      $body = lang('a new correspondence requires your attention.');
      $body .= ' '.lang('details follow').":\n";
      $body .= "\n".lang('protocol number').': '.$doc_info['protocol']."\n";
      $body .= lang('subject').': '.$doc_info['subject']."\n";
      $body .= lang('deadline').': '.$deadline;
      
      $returncode = $send->msg('email',$to,$subject,$body,'','','',$sender);
      
      if (!$returncode) {
	echo '<p><b>send_notification</b>: '.lang("Failed sending message to '%1' #%2 subject='%3', sender='%4' !!!",$to,$userid,htmlspecialchars($subject), $sender)."<br />\n";
	echo '<i>'.$send->err['desc']."</i><br />\n";
	echo lang('This is mostly caused by a not or wrongly configured SMTP server. Notify your administrator.')."</p>\n";
	echo '<p>'.lang('Click %1here%2 to return to Correspondence.','<a href="'.$GLOBALS['egw']->link('/correspondence/').'">','</a>')."</p>\n";
      }
    }
    unset($send);
    
    return $returncode;
  }

  function send_notification_deadline($to_notify,$days,$doc_protocol,$doc_deadline,$doc_subject)
  {
    $user = $GLOBALS['egw_info']['user']['account_id'];
    $sender = $GLOBALS['egw_info']['user']['preferences']['email']['address'];
    if(!is_object($GLOBALS['egw']->send)) {
      $GLOBALS['egw']->send = CreateObject('phpgwapi.send');
    }
    $send = &$GLOBALS['egw']->send;

    foreach($to_notify as $userid) {
      if($GLOBALS['egw']->accounts->get_type($userid) == 'g') {
        continue;
      }
      $userid = intval($userid);
      
      $preferences = CreateObject('phpgwapi.preferences',$userid);
      $GLOBALS['egw']->accounts->get_account_name($userid,$lid,$to_firstname,$to_lastname);
      $to_fullname = $GLOBALS['egw']->common->display_fullname('',$to_firstname,$to_lastname);
      
      $to = $preferences->email_address($userid);
      if (empty($to) || $to[0] == '@' || $to[0] == '$') {
	// we have no valid email-address
	continue;
      }

      if ($days == 1) {
        $subject = lang('your correspondence is due tomorrow')." - ".$doc_subject;
      }
      else {
        $subject = lang('your correspondence is due in %1 days', $days)." - ".$doc_subject;;
      }
      $subject = trim($send->encode_subject($subject));

      $deadline = $this->date2string($doc_deadline);

      $body = lang('a correspondence requires your attention.');
      $body .= ' '.lang('details follow').":\n";
      $body .= "\n".lang('protocol number').': '.$doc_protocol."\n";
      $body .= lang('subject').': '.$doc_subject."\n";
      $body .= lang('deadline').': '.$deadline."\n\n";
 
      $returncode = $send->msg('email',$to,$subject,$body,'','','',$sender);
      
      if (!$returncode) {
	echo '<p><b>send_notification_deadline</b>: '.lang("Failed sending message to '%1' #%2 subject='%3', sender='%4' !!!",$to,$userid,htmlspecialchars($subject), $sender)."<br />\n";
	echo '<i>'.$send->err['desc']."</i><br />\n";
	echo lang('This is mostly caused by a not or wrongly configured SMTP server. Notify your administrator.')."</p>\n";
	echo '<p>'.lang('Click %1here%2 to return to Correspondence.','<a href="'.$GLOBALS['egw']->link('/correspondence/').'">','</a>')."</p>\n";
      }
    }
    unset($send);
    
    return $returncode;
  }

  function get_adv_search($start = -1, $sort = '', $order = '', $allwords, $notwords, $date, $begindate, $enddate, $deadline, $begindl, $enddl, $category, $status, $doc_to, $offset = -1)
  {
    if (! $sort) {
      $sort = "DESC";
    }
    
    if ($order) {
      $orderclause = "ORDER BY $order $sort";
    }
    else {
      $orderclause = "ORDER BY doc_id ASC";
    }

    $whereclause = "WHERE ";
    $empty = 1;

    if ($allwords) {
      $empty = 0;
      $a_words = explode(' ',$allwords);

      $l_word = $a_words[0];
      $whereclause .= "(doc_from LIKE '%$l_word%' OR doc_to LIKE '%$l_word%' OR subject LIKE '%$l_word%' OR description LIKE '%$l_word%' OR id_archive LIKE '%$l_word%')";
      if (count($a_words) > 1) {
	for($i=1;$i<count($a_words);$i++) {
	  $l_word = $a_words[$i];
	  $whereclause .= " AND (doc_from LIKE '%$l_word%' OR doc_to LIKE '%$l_word%' OR subject LIKE '%$l_word%' OR description LIKE '%$l_word%' OR id_archive LIKE ' %$l_word%')";
	}
      }
    }

    if ($doc_to) {
      if ($empty == 0) {
	$whereclause .= " AND ";
      } else {
	$empty = 0;
      }
      $whereclause .= "(doc_to LIKE '%$doc_to%')";
    }

    if ($notwords) {
      if ($empty == 0) {
	$whereclause .= " AND ";
      } else {
	$empty = 0;
      }

      $a_words = explode(' ',$notwords);

      $l_word = $a_words[0];
      $whereclause .= "(doc_from NOT LIKE '%$l_word%' AND doc_to NOT LIKE '%$l_word%' AND subject NOT LIKE '%$l_word%' AND description NOT LIKE '%$l_word%' AND id_archive NOT LIKE '%$l_word%')";
      if (count($a_words) > 1) {
	for($i=1;$i<count($a_words);$i++) {
	  $l_word = $a_words[$i];
	  $whereclause .= " AND (doc_from NOT LIKE '%$l_word%' AND doc_to NOT LIKE '%$l_word%' AND subject NOT LIKE '%$l_word%' AND description NOT LIKE '%$l_word%' AND id_archive NOT LIKE '%$l_word%')";
	}
      }
    }

    if ($date) {
      if ($date != 'any') {
	if ($empty == 0) {
	  $whereclause .= " AND ";
	} else {
	  $empty = 0;
	}
	
	$today = time();

	switch ($date) {
	case 'week':
	  $whereclause .= "startdate>".intval($today-7*24*3600);
	  break;
	  
	case 'month':
	  $whereclause .= "startdate>".intval($today-30*24*3600);
	  break;
	  
	case 'month3':
	  $whereclause .= "startdate>".intval($today-3*30*24*3600);
	  break;

	case 'year':
	  $whereclause .= "startdate>".intval($today-12*30*24*3600);
	  break;

	case 'custom':
	  $this->fix_update_time($begindate);
	  $begin = mktime(0,0,0,$begindate['month'],$begindate['mday'],$begindate['year']) - $GLOBALS['egw']->datetime->tz_offset;
	  $this->fix_update_time($enddate);
	  $end = mktime(0,0,0,$enddate['month'],$enddate['mday'],$enddate['year']) - $GLOBALS['egw']->datetime->tz_offset;
	  $whereclause .= "startdate>".intval($begin)." AND startdate<".intval($end);
	  break;

	default:
	  break;
	}
      }
    }

    if ($deadline) {
      if ($deadline != 'any') {
	if ($empty == 0) {
	  $whereclause .= " AND ";
	} else {
	  $empty = 0;
	}
	
	$today = time();

	switch ($deadline) {
	case 'week':
	  $whereclause .= "deadline>".intval($today-7*24*3600);
	  break;
	  
	case 'month':
	  $whereclause .= "deadline>".intval($today-30*24*3600);
	  break;
	  
	case 'month3':
	  $whereclause .= "deadline>".intval($today-3*30*24*3600);
	  break;

	case 'year':
	  $whereclause .= "deadline>".intval($today-12*30*24*3600);
	  break;

	case 'custom':
	  $this->fix_update_time($begindl);
	  $begin = mktime(0,0,0,$begindl['month'],$begindl['mday'],$begindl['year']) - $GLOBALS['egw']->datetime->tz_offset;
	  $this->fix_update_time($enddl);
	  $end = mktime(0,0,0,$enddl['month'],$enddl['mday'],$enddl['year']) - $GLOBALS['egw']->datetime->tz_offset;
	  $whereclause .= "deadline>".intval($begin)." AND deadline<".intval($end);
	  break;

	default:
	  break;
	}
      }
    }

    if ($category) {
      if ($category != 0) {
	if ($empty == 0) {
	  $whereclause .= " AND ";
	} else {
	  $empty = 0;
	}

	$whereclause .= "category='$category'";
      }
    }   

    if ($status) {
      if ($empty == 0) {
	$whereclause .= " AND ";
      } else {
	$empty = 0;
      }

      if ($status == 'concluded') {
	$whereclause .= " status='".$status."'";
      }
      elseif ($status == 'pending') {
	$whereclause .= " status='pending' AND " . '(deadline>'.time().' OR deadline=0)';
      }
      elseif ($status == 'expired') {
	$whereclause .= " status='pending' AND deadline<".time();
      }
    }

    if ($empty) {
      $sql = "SELECT * FROM phpgw_correspondence $orderclause";
    } else {
      $sql = "SELECT * FROM phpgw_correspondence $whereclause $orderclause";
    }

    if ($offset == -1 && $start == -1) {
      $this->db->query($sql,__LINE__,__FILE__);
    } 
    elseif ($start != -1) {
      $this->db->limit_query($sql,$start,__LINE__,__FILE__);
    }
    else {
      $this->db->limit_query($sql,$start,__LINE__,__FILE__,$offset);
    }

    while ($this->db->next_record()) {
      $data[] = Array(
			  'doc_id'    => $this->db->f('doc_id'),
			  'protocol'  => $this->db->f('id_protocol'),
			  'doc_from'  => $this->db->f('doc_from'),
			  'doc_to'    => $this->db->f('doc_to'),
			  'startdate' => $this->db->f('startdate'),
			  'deadline'  => $this->db->f('deadline'),
			  'owner'     => $this->db->f('owner'),
			  'doc_type'  => $this->db->f('doc_type'),
			  'subject'   => $this->db->f('subject'),
			  'status'    => $this->db->f('status'),
			  'priority'  => $this->db->f('priority')
			  );
    }
    if ($empty) {
      $this->db->query("SELECT count(*) FROM phpgw_correspondence");
    } else {
      $this->db->query("SELECT count(*) FROM phpgw_correspondence $whereclause");
    }

    $this->db->next_record();
    $this->total = $this->db->f(0);

    return $data;
  }
}
?>
